package demo.client;

import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

import demo.client.service.EmployeeService;
import demo.client.service.EmployeeServiceAsync;
import demo.shared.domain.Salary;
import demo.shared.entity.Employee;

public class EmployeeMaintenance extends Composite {

    private static final int LIMIT = 5;

    private final EmployeeServiceAsync employeeService = GWT
            .create(EmployeeService.class);

    private Label titleLabel = new Label("Employee Maintenance");

    private HTML createLink = new HTML("<a href='javascript:;'>create</a>");

    private FlexTable table = new FlexTable();

    private Button prevButton = new Button("Prev");

    private Button nextButton = new Button("Next");

    private NumberFormat salaryNumberFormat = NumberFormat.getFormat("#,##0");

    private int offset = 0;

    public EmployeeMaintenance() {
        super();
        final VerticalPanel panel = new VerticalPanel();
        select(new Delegate<List<Employee>>() {

            @Override
            public void execute(List<Employee> employeeList) {
                initTtileLabel();
                initCreateLink();
                initPrevAndNextButtons();
                initTable();
                updateTable(employeeList);

                HorizontalPanel buttonPanel = new HorizontalPanel();
                buttonPanel.setWidth("100%");
                buttonPanel.add(prevButton);
                buttonPanel.add(nextButton);
                panel.add(titleLabel);
                panel.setSpacing(5);
                panel.add(createLink);
                panel.setCellHorizontalAlignment(createLink,
                        HasHorizontalAlignment.ALIGN_RIGHT);
                panel.add(table);
                panel.add(buttonPanel);
            }
        });

        initWidget(panel);
    }

    private void initTtileLabel() {
        titleLabel.addStyleName("title");
    }

    private void initCreateLink() {
        createLink.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                final CreateDialogBox dialogBox = new CreateDialogBox(
                        new Employee());
                dialogBox.showRelativeTo(createLink);
            }
        });
    }

    private void initPrevAndNextButtons() {
        prevButton.setWidth("100%");
        prevButton.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                offset = Math.max(0, offset - LIMIT);
                select(new Delegate<List<Employee>>() {

                    @Override
                    public void execute(List<Employee> value) {
                        updateTable(value);
                    }
                });
            }
        });

        nextButton.setWidth("100%");
        nextButton.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                offset += LIMIT;
                select(new Delegate<List<Employee>>() {

                    @Override
                    public void execute(List<Employee> value) {
                        updateTable(value);
                    }
                });
            }
        });
    }

    private void initTable() {
        table.addStyleName("table");
        table.setCellSpacing(0);
        table.setWidth("100%");
        setupHeader();
    }

    private void setupHeader() {
        table.setWidget(0, 0, createHeaderColumnLabel("Id"));
        table.setWidget(0, 1, createHeaderColumnLabel("Name"));
        table.setWidget(0, 2, createHeaderColumnLabel("Salary"));
        table.setWidget(0, 3, createHeaderColumnLabel(""));
        table.getRowFormatter().addStyleName(0, "header");
        for (int i = 0; i < table.getCellCount(0); i++) {
            table.getCellFormatter().addStyleName(0, i,
                    "gwtsolutions-FlexTable-ColumnLabelCell");
        }
    }

    private Label createHeaderColumnLabel(String text) {
        Label label = new Label(text);
        label.setWidth("100%");
        label.addStyleName("gwtsolutions-FlexTable-ColumnLabel");
        return label;
    }

    private void updateTable(List<Employee> employeeList) {
        int size = Math.min(LIMIT, employeeList.size());
        int index = 0;
        for (; index < size; index++) {
            int row = index + 1;
            final Employee e = employeeList.get(index);
            table.setWidget(row, 0, createIdCellLabel(e.getId()));
            table.getCellFormatter().addStyleName(row, 0, "idCell");
            table.setWidget(row, 1, createNameCellLabel(e.getName()));
            table.getCellFormatter().addStyleName(row, 1, "nameCell");
            table.setWidget(row, 2, createSalaryCellLabel(e.getSalary()));
            table.getCellFormatter().addStyleName(row, 2, "salaryCell");
            table.setWidget(row, 3, createDeleteAndEditLinkPanel(e));
            table.getCellFormatter().addStyleName(row, 3, "linkCell");
        }
        for (; index < LIMIT; index++) {
            int row = index + 1;
            table.setWidget(index + 1, 0, new Label(" "));
            table.getCellFormatter().addStyleName(row, 0, "idCell");
            table.setWidget(index + 1, 1, new Label(" "));
            table.getCellFormatter().addStyleName(row, 1, "nameCell");
            table.setWidget(index + 1, 2, new Label(" "));
            table.getCellFormatter().addStyleName(row, 2, "salaryCell");
            table.setWidget(index + 1, 3, new Label(" "));
            table.getCellFormatter().addStyleName(row, 3, "linkCell");
        }
        for (int row = 1; row < table.getRowCount(); row++) {
            if ((row % 2) != 0) {
                table.getRowFormatter().removeStyleName(row, "evenRow");
                table.getRowFormatter().addStyleName(row, "oddRow");
            } else {
                table.getRowFormatter().removeStyleName(row, "oddRow");
                table.getRowFormatter().addStyleName(row, "evenRow");
            }
        }

        prevButton.setEnabled(offset >= LIMIT);
        nextButton.setEnabled(employeeList.size() > LIMIT);
    }

    private Label createIdCellLabel(Integer value) {
        String text = value != null ? value.toString() : "";
        Label label = new Label(text);
        label.setWidth("100%");
        return label;
    }

    private Label createNameCellLabel(String value) {
        String text = value != null ? value.toString() : "";
        Label label = new Label(text);
        label.setWidth("100%");
        return label;
    }

    private Label createSalaryCellLabel(Salary value) {
        String text = value != null ? salaryNumberFormat.format(value
                .getValue()) : "";
        Label label = new Label(text);
        label.setWidth("100%");
        label.addStyleName("number");
        return label;
    }

    private Panel createDeleteAndEditLinkPanel(final Employee Employee) {
        HorizontalPanel panel = new HorizontalPanel();
        panel.setSpacing(10);
        panel.add(createDeleteLink(Employee));
        panel.add(createEditLink(Employee));
        return panel;
    }

    private HTML createDeleteLink(final Employee Employee) {
        final HTML link = new HTML("<a href='javascript:;'>delete</a>");
        link.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                DeleteDialogBox dialogBox = new DeleteDialogBox(Employee);
                dialogBox.showRelativeTo(link);
            }
        });
        return link;
    }

    private HTML createEditLink(final Employee Employee) {
        final HTML link = new HTML("<a href='javascript:;'>edit</a>");
        link.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                ConfirmDialogBox dialogBox = new EditDialogBox(Employee);
                dialogBox.showRelativeTo(link);
            }
        });
        return link;
    }

    private void selectAndUpdateTable() {
        select(new Delegate<List<Employee>>() {

            @Override
            void execute(List<Employee> value) {
                updateTable(value);
            }
        });
    }

    private void select(final Delegate<List<Employee>> delegate) {
        employeeService.select(offset, delegate);
    }

    private void insert(Employee Employee, final Delegate<Void> delegate) {
        employeeService.insert(Employee, delegate);
    }

    private void update(Employee Employee, final Delegate<Void> delegate) {
        employeeService.update(Employee, delegate);
    }

    private void delete(Employee Employee, final Delegate<Void> delegate) {
        employeeService.delete(Employee, delegate);
    }

    private abstract class ConfirmDialogBox extends DialogBox {

        protected final Employee employee;

        protected FlexTable flexTable = new FlexTable();

        protected HorizontalPanel buttonPanel = new HorizontalPanel();

        protected TextBox idTextBox = new TextBox();

        protected TextBox nameTextBox = new TextBox();

        protected Label nameErrorLabel = new Label();

        protected TextBox salaryTextBox = new TextBox();

        protected Label salaryErrorLabel = new Label();

        protected ConfirmDialogBox(Employee Employee) {
            super(false);
            this.employee = Employee;

            initTable();
            initButtonPanel();

            VerticalPanel panel = new VerticalPanel();
            panel.add(flexTable);
            panel.add(buttonPanel);
            panel.addStyleName("confirmDialogBox");

            setWidget(panel);
        }

        protected void initTable() {
            idTextBox.setMaxLength(10);
            idTextBox.setVisibleLength(15);
            if (employee.getId() != null) {
                idTextBox.setValue(employee.getId().toString());
            }
            idTextBox.setReadOnly(true);

            nameTextBox.setMaxLength(10);
            nameTextBox.setVisibleLength(15);
            if (employee.getName() != null) {
                nameTextBox.setValue(employee.getName());
            }
            nameErrorLabel.addStyleName("error");

            salaryTextBox.setMaxLength(9);
            salaryTextBox.setVisibleLength(15);
            if (employee.getSalary() != null) {
                salaryTextBox.setValue(employee.getSalary().toString());
            }
            salaryErrorLabel.addStyleName("error");

            flexTable.setText(0, 0, "Id");
            flexTable.setWidget(0, 1, idTextBox);
            flexTable.setText(1, 0, "Name");
            flexTable.setWidget(1, 1, nameTextBox);
            flexTable.setWidget(1, 2, nameErrorLabel);
            flexTable.setText(2, 0, "Salary");
            flexTable.setWidget(2, 1, salaryTextBox);
            flexTable.setWidget(2, 2, salaryErrorLabel);
        }

        protected void initButtonPanel() {
            buttonPanel.setWidth("100%");
            buttonPanel
                    .setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
            buttonPanel.add(createExecuteButton());
            buttonPanel.add(createCloseButton());
        }

        protected Button createExecuteButton() {
            Button button = new Button("Execute");
            return button;
        }

        protected Button createCloseButton() {
            Button button = new Button("Close");
            button.addClickHandler(new ClickHandler() {
                @Override
                public void onClick(ClickEvent event) {
                    hide();
                }
            });
            return button;
        }

        protected boolean validateAndSetName() {
            nameErrorLabel.setText("");
            if (nameTextBox.getValue() == null
                    || nameTextBox.getValue().isEmpty()) {
                nameErrorLabel.setText("must be non-empty.");
                return false;
            }
            employee.setName(nameTextBox.getValue());
            return true;
        }

        protected boolean validateAndSetSalary() {
            salaryErrorLabel.setText("");
            if (salaryTextBox.getValue() == null) {
                return false;
            }
            Double salary;
            try {
                salary = salaryNumberFormat.parse(salaryTextBox.getValue());
            } catch (NumberFormatException e) {
                salaryErrorLabel.setText("must be number.");
                return false;
            }
            employee.setSalary(new Salary(salary.longValue()));
            return true;
        }
    }

    private class CreateDialogBox extends ConfirmDialogBox {

        private CreateDialogBox(Employee Employee) {
            super(Employee);
            setText("Confirm : Create");
        }

        @Override
        protected void initTable() {
            super.initTable();
            idTextBox.setVisible(false);
        }

        @Override
        protected Button createExecuteButton() {
            Button button = super.createExecuteButton();
            button.addClickHandler(new ClickHandler() {

                @Override
                public void onClick(ClickEvent event) {
                    if (!validateAndSetName() | !validateAndSetSalary()) {
                        return;
                    }

                    EmployeeMaintenance.this.insert(employee,
                            new Delegate<Void>() {

                                @Override
                                void execute(Void value) {

                                    EmployeeMaintenance.this
                                            .selectAndUpdateTable();
                                }
                            });
                    hide();
                }
            });
            return button;
        }
    }

    private class EditDialogBox extends ConfirmDialogBox {

        private EditDialogBox(Employee Employee) {
            super(Employee);
            setText("Confirm : Edit");
        }

        @Override
        protected Button createExecuteButton() {
            Button button = super.createExecuteButton();
            button.addClickHandler(new ClickHandler() {

                @Override
                public void onClick(ClickEvent event) {
                    if (!validateAndSetName()) {
                        return;
                    }
                    if (!validateAndSetSalary()) {
                        return;
                    }

                    EmployeeMaintenance.this.update(employee,
                            new Delegate<Void>() {

                                @Override
                                void execute(Void value) {

                                    EmployeeMaintenance.this
                                            .selectAndUpdateTable();
                                }
                            });
                    hide();
                }
            });
            return button;
        }

    }

    private class DeleteDialogBox extends ConfirmDialogBox {

        private DeleteDialogBox(Employee Employee) {
            super(Employee);
            setText("Confirm : Delete");
        }

        @Override
        protected void initTable() {
            super.initTable();
            nameTextBox.setReadOnly(true);
            salaryTextBox.setReadOnly(true);
        }

        @Override
        protected Button createExecuteButton() {
            Button button = super.createExecuteButton();
            button.addClickHandler(new ClickHandler() {

                @Override
                public void onClick(ClickEvent event) {
                    EmployeeMaintenance.this.delete(employee,
                            new Delegate<Void>() {

                                @Override
                                void execute(Void value) {

                                    EmployeeMaintenance.this
                                            .selectAndUpdateTable();
                                }
                            });
                    hide();
                }
            });
            return button;
        }
    }

}
